package ru.dbotthepony.mc.otm.block.entity

import net.minecraft.core.BlockPos
import net.minecraft.world.level.block.entity.BlockEntity
import net.minecraft.world.level.block.entity.BlockEntityType
import net.minecraft.world.level.block.state.BlockState
import ru.dbotthepony.mc.otm.core.component1
import ru.dbotthepony.mc.otm.core.component2
import ru.dbotthepony.mc.otm.core.component3
import ru.dbotthepony.mc.otm.core.position
import ru.dbotthepony.mc.otm.network.BlockEntitySyncPacket
import ru.dbotthepony.mc.otm.network.FieldSynchronizer
import ru.dbotthepony.mc.otm.network.WorldNetworkChannel

abstract class SynchronizedBlockEntity(p_155228_: BlockEntityType<*>, p_155229_: BlockPos, p_155230_: BlockState) : BlockEntity(p_155228_, p_155229_, p_155230_) {
	val synchronizer = FieldSynchronizer()

	init {
		synchronizer.defaultEndpoint.markUnused()
	}

	open val synchronizationRadius: Double = 32.0

	// TODO: Optimize, if required
	fun synchronizeToPlayers() {
		if (synchronizationRadius <= 0.0 || synchronizer.isEmpty) {
			return
		}

		synchronizer.observe()

		val server = level?.server ?: return

		if (server.playerCount <= 0) {
			return
		}

		val (xi, yi, zi) = blockPos
		val bx = xi + 0.5
		val by = yi + 0.5
		val bz = zi + 0.5

		val double = synchronizationRadius * synchronizationRadius

		server.playerList.players.stream().filter {
			if (!it.isAlive || it.level != level) {
				return@filter false
			}

			val (x, y, z) = it.position

			(x - bx) * (x - bx) +
			(y - by) * (y - by) +
			(z - bz) * (z - bz) <= double
		}.forEach {
			val payload = synchronizer.computeEndpointFor(it).collectNetworkPayload()

			if (payload != null) {
				WorldNetworkChannel.send(it, BlockEntitySyncPacket(blockPos, payload.array, payload.length))
			}
		}
	}

}
